@@ -21,6 +21,7 @@ package org.apache.fury;
 
 import java.util.function.Consumer;
 import java.util.function.Function;
+import org.apache.fury.resolver.ClassChecker;
 import org.apache.fury.serializer.Serializer;
 import org.apache.fury.serializer.SerializerFactory;
 
@@ -65,5 +66,10 @@ public abstract class AbstractThreadSafeFury implements ThreadSafeFury {
     processCallback(fury -> fury.setSerializerFactory(serializerFactory));
   }
 
+  @Override
+  public void setClassChecker(ClassChecker classChecker) {
+    processCallback(fury -> fury.getClassResolver().setClassChecker(classChecker));
+  }
+
   protected abstract void processCallback(Consumer<Fury> callback);
 }
