@@ -398,10 +398,18 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
     return visitFury(f -> f.getClassResolver().isCollection(TypeUtils.getRawType(typeRef)));
   }
 
+  protected boolean useCollectionSerialization(Class<?> type) {
+    return visitFury(f -> f.getClassResolver().isCollection(TypeUtils.getRawType(type)));
+  }
+
   protected boolean useMapSerialization(TypeRef<?> typeRef) {
     return visitFury(f -> f.getClassResolver().isMap(TypeUtils.getRawType(typeRef)));
   }
 
+  protected boolean useMapSerialization(Class<?> type) {
+    return visitFury(f -> f.getClassResolver().isMap(TypeUtils.getRawType(type)));
+  }
+
   /**
    * Whether the provided type should be taken as final. Although the <code>clz</code> can be final,
    * the method can still return false. For example, we return false in meta share mode to write
@@ -491,6 +499,13 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
           serializerClass = Serializer.class;
         }
       }
+      if (useCollectionSerialization(cls)
+          && !AbstractCollectionSerializer.class.isAssignableFrom(serializerClass)) {
+        serializerClass = AbstractCollectionSerializer.class;
+      } else if (useMapSerialization(cls)
+          && !AbstractMapSerializer.class.isAssignableFrom(serializerClass)) {
+        serializerClass = AbstractMapSerializer.class;
+      }
       TypeRef<? extends Serializer> serializerTypeRef = TypeRef.of(serializerClass);
       Expression fieldTypeExpr = getClassExpr(cls);
       // Don't invoke `Serializer.newSerializer` here, since it(ex. ObjectSerializer) may set itself
