@@ -191,6 +191,12 @@ public class Serializers {
       this.typeId = typeId;
     }
 
+    public CrossLanguageCompatibleSerializer(
+        Fury fury, Class<T> cls, short typeId, boolean needToWriteRef, boolean immutable) {
+      super(fury, cls, needToWriteRef, immutable);
+      this.typeId = typeId;
+    }
+
     @Override
     public short getXtypeId() {
       return typeId;
@@ -274,6 +280,11 @@ public class Serializers {
       super(fury, StringBuilder.class);
     }
 
+    @Override
+    public StringBuilder copy(StringBuilder origin) {
+      return new StringBuilder(origin);
+    }
+
     @Override
     public StringBuilder read(MemoryBuffer buffer) {
       return new StringBuilder(stringSerializer.readJavaString(buffer));
@@ -292,6 +303,11 @@ public class Serializers {
       super(fury, StringBuffer.class);
     }
 
+    @Override
+    public StringBuffer copy(StringBuffer origin) {
+      return new StringBuffer(origin);
+    }
+
     @Override
     public StringBuffer read(MemoryBuffer buffer) {
       return new StringBuffer(stringSerializer.readJavaString(buffer));
@@ -303,7 +319,7 @@ public class Serializers {
     }
   }
 
-  public static final class BigDecimalSerializer extends Serializer<BigDecimal> {
+  public static final class BigDecimalSerializer extends ImmutableSerializer<BigDecimal> {
     public BigDecimalSerializer(Fury fury) {
       super(fury, BigDecimal.class);
     }
@@ -328,7 +344,7 @@ public class Serializers {
     }
   }
 
-  public static final class BigIntegerSerializer extends Serializer<BigInteger> {
+  public static final class BigIntegerSerializer extends ImmutableSerializer<BigInteger> {
     public BigIntegerSerializer(Fury fury) {
       super(fury, BigInteger.class);
     }
@@ -359,6 +375,11 @@ public class Serializers {
       buffer.writeBoolean(value.get());
     }
 
+    @Override
+    public AtomicBoolean copy(AtomicBoolean origin) {
+      return new AtomicBoolean(origin.get());
+    }
+
     @Override
     public AtomicBoolean read(MemoryBuffer buffer) {
       return new AtomicBoolean(buffer.readBoolean());
@@ -376,6 +397,11 @@ public class Serializers {
       buffer.writeInt32(value.get());
     }
 
+    @Override
+    public AtomicInteger copy(AtomicInteger origin) {
+      return new AtomicInteger(origin.get());
+    }
+
     @Override
     public AtomicInteger read(MemoryBuffer buffer) {
       return new AtomicInteger(buffer.readInt32());
@@ -393,6 +419,11 @@ public class Serializers {
       buffer.writeInt64(value.get());
     }
 
+    @Override
+    public AtomicLong copy(AtomicLong origin) {
+      return new AtomicLong(origin.get());
+    }
+
     @Override
     public AtomicLong read(MemoryBuffer buffer) {
       return new AtomicLong(buffer.readInt64());
@@ -410,13 +441,18 @@ public class Serializers {
       fury.writeRef(buffer, value.get());
     }
 
+    @Override
+    public AtomicReference copy(AtomicReference origin) {
+      return new AtomicReference(fury.copyObject(origin.get()));
+    }
+
     @Override
     public AtomicReference read(MemoryBuffer buffer) {
       return new AtomicReference(fury.readRef(buffer));
     }
   }
 
-  public static final class CurrencySerializer extends Serializer<Currency> {
+  public static final class CurrencySerializer extends ImmutableSerializer<Currency> {
     public CurrencySerializer(Fury fury) {
       super(fury, Currency.class);
     }
@@ -434,7 +470,7 @@ public class Serializers {
   }
 
   /** Serializer for {@link Charset}. */
-  public static final class CharsetSerializer<T extends Charset> extends Serializer<T> {
+  public static final class CharsetSerializer<T extends Charset> extends ImmutableSerializer<T> {
     public CharsetSerializer(Fury fury, Class<T> type) {
       super(fury, type);
     }
@@ -448,7 +484,7 @@ public class Serializers {
     }
   }
 
-  public static final class URISerializer extends Serializer<java.net.URI> {
+  public static final class URISerializer extends ImmutableSerializer<java.net.URI> {
 
     public URISerializer(Fury fury) {
       super(fury, URI.class);
@@ -465,7 +501,7 @@ public class Serializers {
     }
   }
 
-  public static final class RegexSerializer extends Serializer<Pattern> {
+  public static final class RegexSerializer extends ImmutableSerializer<Pattern> {
     public RegexSerializer(Fury fury) {
       super(fury, Pattern.class);
     }
@@ -484,7 +520,7 @@ public class Serializers {
     }
   }
 
-  public static final class UUIDSerializer extends Serializer<UUID> {
+  public static final class UUIDSerializer extends ImmutableSerializer<UUID> {
 
     public UUIDSerializer(Fury fury) {
       super(fury, UUID.class);
@@ -502,7 +538,7 @@ public class Serializers {
     }
   }
 
-  public static final class ClassSerializer extends Serializer<Class> {
+  public static final class ClassSerializer extends ImmutableSerializer<Class> {
     public ClassSerializer(Fury fury) {
       super(fury, Class.class);
     }
@@ -525,7 +561,7 @@ public class Serializers {
    * serializable or class registration checks.
    */
   // Use a separate serializer to avoid codegen for emtpy object.
-  public static final class EmptyObjectSerializer extends Serializer<Object> {
+  public static final class EmptyObjectSerializer extends ImmutableSerializer<Object> {
 
     public EmptyObjectSerializer(Fury fury) {
       super(fury, Object.class);
