@@ -22,8 +22,6 @@ package org.apache.fury.serializer.collection;
 import java.util.Collection;
 import org.apache.fury.Fury;
 import org.apache.fury.memory.MemoryBuffer;
-import org.apache.fury.resolver.ClassInfo;
-import org.apache.fury.resolver.ClassResolver;
 
 /** Base serializer for all java collections. */
 @SuppressWarnings({"unchecked", "rawtypes"})
@@ -65,20 +63,6 @@ public class CollectionSerializer<T extends Collection> extends AbstractCollecti
     return (T) newCollection;
   }
 
-  public void copyElements(T originCollection, Collection newCollection) {
-    ClassResolver classResolver = fury.getClassResolver();
-    for (Object element : originCollection) {
-      if (element != null) {
-        ClassInfo classInfo =
-            classResolver.getClassInfo(element.getClass(), elementClassInfoHolder);
-        if (!classInfo.getSerializer().isImmutable()) {
-          element = fury.copyObject(element, classInfo.getClassId());
-        }
-      }
-      newCollection.add(element);
-    }
-  }
-
   @Override
   public T read(MemoryBuffer buffer) {
     Collection collection = newCollection(buffer);
